/***********************************************\
|                                               |
| Input Handler Version 1.0                     |
|                                               |
| Coded by slip - slip@ice-d.com                |
| www.ice-d.com                                 |
|                                               |
| Description:                                  |
| ------------                                  |
| Uses 166 bytes of memory to hold the delay    |
| times for each key press.                     |
| This uses a global 'CSPF' which holds the     |
| current loop time. This is added to the time  |
| holder array to keep track of the time the key|
| has been down.								|
|                                               |
| This code requires CSPF be the Timer value at |
| the end of the loop, then the timer must be   |
| set to zero. The timer must also be at        |
| frequecy 1024, otherwise change the multiple  |
| of the key presses which is currently (1<<14).|
|                                               |
| In order to keep things simple just follow the|
| above and each prorgam loop call UpdateKeys() |
| and you can check (FREG_KEYS & KEY_KEY) being |
| true the key is on and its repeat time has    |
| been met.                                     |
|                                               |
\***********************************************/
/*
How to use:
-----------
File setup and compiling
-------------------------
Include input.h and everything should work!
the gba.h provided with this includes:

	#define CODE_IN_IWRAM __attribute__ ((section (".iwram"), long_call))
	#define CODE_IN_EWRAM __attribute__ ((section (".ewram"), long_call))
	#define CODE_IN_ROM __attribute__ ((section (".text"), long_call))
	#define IN_IWRAM __attribute__ ((section (".iwram")))
	#define IN_ROM __attribute__ ((section (".text")))
	#define IN_EWRAM __attribute__ ((section (".ewram")))

Add this to your gba.h or replace your gba.h with the one provided.

To compile 

	gcc -c -o input.o input.cpp -mthumb-interwork -O1
or
	gcc -c -o input.o input.cpp -O1
if you don't want arm-thumb interworking. remove -O1 if you
don't want optimization. Note: using -O2 or -O3, you may need to move
ResetTimerCSPF() into another file, such as your main file. For some
reason the higher optimization causes this function to work in correctly.

If you are happy with the default settings or are at least happy to change
the repeat times yourself via the arrays (see below) then just link
one of the object files precompiled with your project.
	inputIW.o		for arm-thumb interworking
	input.o			for no interworking
both are compiled with the -O3 switch

batch files are included to aid in compiling.

Coding
-----------
Before the input handler will work corectly you must call:
	SetupInputTimer();
	SetupInput();

SetupInputTimer() will setup a timer which will be used by the handler.
	The timer will be reset each loop after the value of it
	has been recorded. It is recorded to CSPF (Current Seconds Per Frame)
You can change the timer that will be used in input.h by changing the
definitions of REG_TM_INPUT_CNT and REG_TM_INPUT_D. The default is timer 3.

SetupInput() will setup the times of each key press and delay. You can change
these in this file. "Inital repeat times" is the delay before a key
it repeated the first time, after this first repeat the "Repeat times"
as delay times.
	You can change the repeat times at any point:
	The Arrays KEYREPEATTIMEINIT and KEYREPEATTIME defined in this file
	hold the Inital repeat times and Repeat times respectively. The arrays
	are setup as follow:
		[0] A
		[1] B
		[2] SELECT
		[3] START
		[4] RIGHT
		[5] LEFT
		[6] UP
		[7] DOWN
		[8] R
		[9] L
	For example:
		To change the repeat time of button B at some point you would change
		KEYREPEATTIME[1]. If you are using the default timer settings then
		the time can be determined by.
			KEYREPEATTIME[1] = (u32)(TIME * (1<<14));
		where time is the time in seconds.

	Note: if you want to change the timer frequency to 64 or 256, then the multiple 
	will need to be changed. The Multiple is (1<<14) for 1024.

Every Frame
------------
Every time you want FREG_KEYS updated call
	UpdateKeys();
And call
	ResetTimerCSPF();
once each frame to reset the timer and update CSPF;
Its a good idea to call UpdateKeys(); right before ResetTimerCSPF();
otherwise CSPF wont be the correct frame time when entering UpdateKeys();
which can cause extended delay times.

Whenever FREG_KEYS needs to be updated call
	UpdateKeys();
But keep in mind that the CSPF is used. 

Conclusion
-----------
Call:
	SetupInputTimer();
	SetupInput();
to setup the handler.

Call:
	UpdateKeys();
	ResetTimerCSPF();
to update FREG_KEYS.

Call:
	UpdateKeys();
to update FREG_KEYS at any time using the current CSPF

any questions or problems email
slip@ice-d.com
or
contact@ice-d.com
*/
#include "input.h"

	u32 CSPF;
	u16 FREG_KEYS;					//
	u16 KEYFLAGS;
	u16 KEYHOLDFLAGS;
	u32 KEYHOLDTIMER[10];
	u32 KEYREPEATTIME[10];			//Repeat times
	u32 KEYREPEATTIMEINIT[10];		//Inital repeat time (delay before repeating)
	u32 KEYREPEATTIMETEST[10];	

void SetupInput()
{
	//Leave these
	FREG_KEYS = 0;
	KEYFLAGS = 0;
	KEYHOLDFLAGS = 0;
	KEYHOLDTIMER[0] = 0;
	KEYHOLDTIMER[1] = 0;
	KEYHOLDTIMER[2] = 0;
	KEYHOLDTIMER[3] = 0;
	KEYHOLDTIMER[4] = 0;
	KEYHOLDTIMER[5] = 0;
	KEYHOLDTIMER[6] = 0;
	KEYHOLDTIMER[7] = 0;
	KEYHOLDTIMER[8] = 0;
	KEYHOLDTIMER[9] = 0;

	//Repeat times. Change the floating point value. The floating point time is in seconds
	KEYREPEATTIME[0] = (u32)(0.1f * (1<<14));	//A
	KEYREPEATTIME[1] = (u32)(0.5f * (1<<14));		//B
	KEYREPEATTIME[2] = (u32)(1.0f * (1<<14));		//SELECT
	KEYREPEATTIME[3] = (u32)(1.0f * (1<<14));		//START
	KEYREPEATTIME[4] = (u32)(0.03f * (1<<14));		//RIGHT
	KEYREPEATTIME[5] = (u32)(0.03f * (1<<14));		//LEFT
	KEYREPEATTIME[6] = (u32)(0.03f * (1<<14));		//UP
	KEYREPEATTIME[7] = (u32)(0.03f * (1<<14));		//DOWN
	KEYREPEATTIME[8] = (u32)(0.03f * (1<<14));		//R
	KEYREPEATTIME[9] = (u32)(0.03f * (1<<14));		//L

	//Inital repeat times. Change the floating point value. The floating point time is in seconds
	KEYREPEATTIMEINIT[0] = (u32)(0.2f * (1<<14));		//A
	KEYREPEATTIMEINIT[1] = (u32)(0.5f * (1<<14));		//B
	KEYREPEATTIMEINIT[2] = (u32)(1.0f * (1<<14));		//SELECT
	KEYREPEATTIMEINIT[3] = (u32)(1.0f * (1<<14));		//START
	KEYREPEATTIMEINIT[4] = (u32)(0.1f * (1<<14));		//RIGHT
	KEYREPEATTIMEINIT[5] = (u32)(0.1f * (1<<14));		//LEFT
	KEYREPEATTIMEINIT[6] = (u32)(0.1f * (1<<14));		//UP
	KEYREPEATTIMEINIT[7] = (u32)(0.1f * (1<<14));		//DOWN
	KEYREPEATTIMEINIT[8] = (u32)(0.1f * (1<<14));		//R
	KEYREPEATTIMEINIT[9] = (u32)(0.1f * (1<<14));		//L

	KEYREPEATTIMETEST[0] = KEYREPEATTIMEINIT[0];
	KEYREPEATTIMETEST[1] = KEYREPEATTIMEINIT[1];
	KEYREPEATTIMETEST[2] = KEYREPEATTIMEINIT[2];
	KEYREPEATTIMETEST[3] = KEYREPEATTIMEINIT[3];
	KEYREPEATTIMETEST[4] = KEYREPEATTIMEINIT[4];
	KEYREPEATTIMETEST[5] = KEYREPEATTIMEINIT[5];
	KEYREPEATTIMETEST[6] = KEYREPEATTIMEINIT[6];
	KEYREPEATTIMETEST[7] = KEYREPEATTIMEINIT[7];
	KEYREPEATTIMETEST[8] = KEYREPEATTIMEINIT[8];
	KEYREPEATTIMETEST[9] = KEYREPEATTIMEINIT[9];
}

void UpdateKeys()
{
	for(register int ck=0; ck < 10; ++ck)
	{
		register int ckt = 1<<ck;
		//if the key is down but wasn't down. OR, the key timer >= timer tester
		if((!(*KEYS & ckt) && !(KEYFLAGS & ckt)) || (KEYHOLDTIMER[ck] >= KEYREPEATTIMETEST[ck])){
			//if the key was already down (condision 2 was true) then set ther timer tester to the REPEAT TIME
			if(KEYFLAGS & ckt)
				KEYREPEATTIMETEST[ck] = KEYREPEATTIME[ck];

			if(!(KEYHOLDTIMER[ck] >= KEYREPEATTIMETEST[ck]))
				KEYFLAGS = KEYFLAGS ^ ckt;
			else
				KEYHOLDTIMER[ck] = 0;

			/////////////////////
			//DO STUFF HERE//////
			/////////////////////
			FREG_KEYS |= ckt;			//Set the flags to check

		}else
		{
			if((*KEYS & ckt) && (KEYFLAGS & ckt))
			{
				KEYFLAGS = KEYFLAGS ^ ckt;
				KEYHOLDTIMER[ck] = 0;
				KEYREPEATTIMETEST[ck] = KEYREPEATTIMEINIT[ck];
				//if(FREG_KEYS & ckt)			//If the flag for the key is set we have to unset it
//					FREG_KEYS ^= ckt;
			}
			if(FREG_KEYS & ckt)			//If the flag for the key is set we have to unset it
					FREG_KEYS ^= ckt;
		}

			if(!(*KEYS & ckt) && (KEYFLAGS & ckt))			// Add to the timer
				KEYHOLDTIMER[ck] += CSPF;
	}

}

void ResetTimerCSPF()
{
	CSPF = REG_TM_INPUT_D;
	//reset the timmer
	REG_TM_INPUT_CNT ^= TIME_ENABLE;
	REG_TM_INPUT_D = 0;
	REG_TM_INPUT_CNT ^= TIME_ENABLE;
}

void SetupInputTimer()
{
	REG_TM_INPUT_CNT = TIME_FREQUENCY_1024 | TIME_ENABLE;
}
